﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Diagnostics;
using ProtoBuf;

namespace Utility
{
    [ProtoContract]
    public class Date
    {
        [ProtoMember(1)]
        DateTime innerDateTime = DateTime.MinValue;

        public Date(DateTime dateTime)
        {
            this.innerDateTime = dateTime.Date;
        }

        public Date()
        {
        }

        public static Date MinValue
        {
            get
            {
                return new Date(DateTime.MinValue);
            }
        }

        public static Date MaxValue
        {
            get
            {
                return new Date(DateTime.MaxValue);
            }
        }

        public static Date Today
        {
            get
            {
                return new Date(DateTime.Now);
            }
        }

        public static Date FromDateTime(DateTime dateTime)
        {
            return new Date(dateTime);
        }

        public DateTime InnerDateTime
        {
            get
            {
                return innerDateTime;
            }
        }

        /// <summary>
        /// 在开始日期和结束日期中
        /// </summary>
        /// <param name="start"></param>
        /// <param name="end"></param>
        /// <returns></returns>
        public bool IsBetween(Date start, Date end)
        {
            if(this < start)
            {
                return false;
            }

            if(this > end)
            {
                return false;
            }

            return true;
        }

        public int ToInt()
        {
            return int.Parse(ToString());
        }

        /// <summary>
        /// 获得新日期
        /// </summary>
        /// <param name="date"></param>
        /// <returns></returns>
        public Date AddDays(int date)
        {
            DateTime dateTime = innerDateTime.AddDays(date);
            return new Date(dateTime);
        }

        public Date AddMonths(int month)
        {
            DateTime dateTime = innerDateTime.AddMonths(month);
            return new Date(dateTime);
        }

        public override bool Equals(object obj)
        {
            if (obj is Date)
            {
                return innerDateTime == ((Date)obj).innerDateTime;
            }
            return false;
        }

        public override int GetHashCode()
        {
            return ToString().GetHashCode();
        }

        public override string ToString()
        {
            return innerDateTime.ToYMDString();
        }

        public static bool operator ==(Date d1, Date d2)
        {
            bool d1_null = (d1 as object) == null;
            bool d2_null = (d2 as object) == null;

            if (d1_null && d2_null)
                return true;

            if ((d1_null && !d2_null) || (d2_null && !d1_null))
                return false;

            return d1.innerDateTime == d2.innerDateTime;

        }

        public static bool operator !=(Date d1, Date d2)
        {
            return !(d1 == d2);
        }

        public static bool operator <(Date t1, Date t2)
        {
            return t1.innerDateTime < t2.innerDateTime;
        }

        public static bool operator <=(Date t1, Date t2)
        {
            return t1.innerDateTime <= t2.innerDateTime;
        }

        public static bool operator >(Date t1, Date t2)
        {
            return t1.innerDateTime > t2.innerDateTime;
        }

        public static bool operator >=(Date t1, Date t2)
        {
            return t1.innerDateTime >= t2.innerDateTime;
        }

        public static Int32 operator - (Date t1, Date t2)
        {
            return (Int32)(t1.innerDateTime - t2.innerDateTime).TotalDays;
        }
    }
}
